######################################################################################################
##  NoitaSpinCythonTest Module to test Cython from NoitaSpin
##
##                      To build, run python setup.py build_ext --inplace.
##                      Then do import NoitaSpinCythonTest and use the imported function.
##
##  Copyright           (C) Tietonoita / Maria Pinjanainen 2013 - 2016.
##
##  Algorithm           The reason why we have separate routines to read (GetSerialNumbers)
##                      and check the serial numbers (VerifySerials) is that privileges are
##                      needed to read the serial numbers. This is done running the
##                      GetSerialNumbers as a privileged process. We cannot verify the serial
##                      numbers in the same privileges process/routine and pass the result of
##                      this comparision back to the NoitaSpin process: this communication
##                      would be too easy to hack. To instead, the serial numbers are passed back
##                      (encrypted) and the verifying is done in non-privileged mode, in the
##                      NoitaSpin process.

import os

cdef char* Copyright = "Copyright (C) Tietonoita / Maria Pinjanainen 2013 - 2016."

def CythonTest():
    return "Call to cython testfunction successful."

### Simple encryption
###
### OBS an exception in the coding: if the result of the encryption would be DEL (127), then
### the character is left unmodified. This because DEL is not handled well in Windows.

cdef char* cKey = "WellDone. NowBuyALegalCopy."     # Should be an *unreadable* string !

def Transmogrify(InString):
    Key = <object> cKey
    OutString = ""

    for Index in range(len(InString)):
        if   (ord(InString[Index]) >= 0x60 and ord(InString[Index]) <= 0x7F):
            if ord(InString[Index]) ^ (ord(Key[Index % len(Key)]) & 0x1F) != 127:
                OutString = OutString + chr(ord(InString[Index]) ^ (ord(Key[Index % len(Key)]) & 0x1F))
            else:
                OutString = OutString + InString[Index]
        elif (ord(InString[Index]) >= 0x30 and ord(InString[Index]) <= 0x3F) or \
             (ord(InString[Index]) >= 0x40 and ord(InString[Index]) <= 0x4F):
            OutString = OutString + chr(ord(InString[Index]) ^ (ord(Key[Index % len(Key)]) & 0x0F))
        elif (ord(InString[Index]) >= 0x28 and ord(InString[Index]) <= 0x2F) or \
             (ord(InString[Index]) >= 0x50 and ord(InString[Index]) <= 0x57):
            OutString = OutString + chr(ord(InString[Index]) ^ (ord(Key[Index % len(Key)]) & 0x07))
        elif (ord(InString[Index]) >= 0x20 and ord(InString[Index]) <= 0x23) or \
             (ord(InString[Index]) >= 0x58 and ord(InString[Index]) <= 0x5B):
            OutString = OutString + chr(ord(InString[Index]) ^ (ord(Key[Index % len(Key)]) & 0x03))
        else:
            OutString = OutString + InString[Index]

    return OutString


### The following is license handling and checking for Linux.

# License for Tietonoita's vilijonkka  Thinkpad T60

cdef char* board_serial = 'J=<)7M>7>9>?300'
cdef char* chassis_serial = 'Sj Na#Hlblkk#@l!N*@)L*'
cdef char* product_serial = 'Sj Na#Hlblkk#@l!N*@)L*'
cdef char* product_uuid = '76<<4=>5+0:?7/5<1<(71<5*09>05<<<?>57'


# License for Tietonoita's vilijonkka  Thinkpad T60

#cdef char* board_serial = 'QC<P=8=4?Y8'
#cdef char* chassis_serial = 'Ijx Eyolbalcr'
#cdef char* product_serial = 'K6<H6L7'
#cdef char* product_uuid = '37N9@J64+46LF/48BN(>2J@*99:4G540I=0?'

# Error license

#cdef char* product_uuid = '37N9@J64+46LF/48BN(>2J@*99:4G541I=0?'       # Err license for testing


def SourceLicIDpairs():
    return (("/sys/class/dmi/id/board_serial",   <object> board_serial), \
            ("/sys/class/dmi/id/chassis_serial", <object> chassis_serial), \
            ("/sys/class/dmi/id/product_serial", <object> product_serial), \
            ("/sys/class/dmi/id/product_uuid",   <object> product_uuid))

def GetPrivIDs(Crypt=True):
    for SourceLicIDpair in SourceLicIDpairs():
        InFile = open(SourceLicIDpair[0], "r")
        String = ""

        while InFile:
            Line = InFile.readline()
            if not Line: break
            String = String+Line[:-1]

        if Crypt:
            print "'"+Transmogrify(String)+"', ",
        else:
            print "'"+String+"', ",

        InFile.close()

def VerifyPrivIDs(BoardSerial, ChassisSerial, ProductSerial, ProductUUID):

    print "NoitaSpinCythonTest: read BoardSerial, ChassisSerial, ProductSerial, ProductUUID: ", BoardSerial, ChassisSerial, ProductSerial, ProductUUID
    print "                     license info:                                                ", SourceLicIDpairs()[0][1], SourceLicIDpairs()[1][1], SourceLicIDpairs()[2][1], SourceLicIDpairs()[3][1]

    if SourceLicIDpairs()[0][1] == BoardSerial   and \
       SourceLicIDpairs()[1][1] == ChassisSerial and \
       SourceLicIDpairs()[2][1] == ProductSerial and \
       SourceLicIDpairs()[3][1] == ProductUUID: return True
    else:                                       return False

### The following is license handling and checking for Windows.

# License for Tietonoita's NS virtual Windows machine running on mymmeli:


#cdef char*  OS_Name = 'Microsoft Windows 7 Home Premium'
#cdef char*  System_Model = 'VirtualBox'
#cdef char*  OS_Build_Type = 'Multiprocessor Free'
#cdef char*  OS_Version = '6.1.7601 Service Pack 1 Build 7601'
#cdef char*  System_Type = 'x64-based PC'
#cdef char*  BIOS_Version = 'innotek GmbH VirtualBox, 1.12.2006'
#cdef char*  Processors = '1 Processor(s) Installed.                           [01]: Intel64 Family 6 Model 60 Stepping 3 GenuineIntel ~3399 Mhz', 
#cdef char*  Product_ID = '00359-OEM-8703854-85042'
#cdef char*  System_Manufacturer = 'innotek GmbH'

cdef char*  OS_Name = 'Jlo~k|acz Qfyfznr 2#Icnj Q|rheyi'
cdef char*  System_Model = 'Ql~xqnbGax'
# old crypt cdef char*  OS_Build_Type = 'Jp`xm⌂|jme}|xp!Osi`'
cdef char*  OS_Build_Type = 'Jp`xmp|jme}|xp!Osi`'
cdef char*  OS_Version = '1+=*39>4"Sk}akv|!Tddj 2#Blg{a ;2??'
cdef char*  System_Type = 'o38)fn}`j VL'
cdef char*  BIOS_Version = '~kbcpje!ImlG#T|kuydkCc{+ 8(67*>4?8'
# old crypt cdef char*  Processors = '6!T~klkv}o|/d+!@o⌂qfm`fk.!"#!   #"!" "##"!!! !#! ## Z>6]6 Maz`b6:#Acxpmu!1!Alkuu"15 Wpj~ugni#4"B|oylidEm{uu"i6?5=#Cmt'
cdef char*  Processors = '6!T~klkv}o|/d+!@osqfm`fk.!"#!   #"!" "##"!!! !#! ## Z>6]6 Maz`b6:#Acxpmu!1!Alkuu"15 Wpj~ugni#4"B|oylidEm{uu"i6?5=#Cmt'
cdef char*  Product_ID = '75?9=*A@C-6871=<5)=2181'
cdef char*  System_Manufacturer = '~kbcpje!ImlG'


LicDict = {'OS Name': <object> OS_Name, 
           'System Model': <object> System_Model, 
           'OS Build Type': <object> OS_Build_Type, 
           'OS Version': <object> OS_Version, 
           'System Type': <object> System_Type, 
           'BIOS Version': <object> BIOS_Version, 
           'Processor(s)': <object> Processors, 
           'Product ID': <object> Product_ID, 
           'System Manufacturer': <object> System_Manufacturer}


def VerifyWindowsIDs():
    Lines = os.popen2("SYSTEMINFO")[1].read().split("\n") # Read system info and split into list of lines.
    IDdict = {}                                     # Will contain system info as...
    Key = ""                                        # ...key:value...
    Value = ""                                      # pairs.

    for Line in Lines:
        if Line != "" and Line[0] != " ":           # If the line contains a key (== no leading space):
            if Key != "": IDdict[Key] = Value.strip()   # Put ealier key:value pair into dictionary
            Key = Line.strip().split(":")[0]        # Use string before first ':' as key (strip leading and trailing white space.
            Value = ":".join(Line.split(":")[1:])   # The rest of the line is value.
        else:
            Value = Value + Line                    # Line with leading space: all value, append to earlier value string.

    File = open("IDfile.txt", "w")

    for Key in LicDict.keys():
        try:
            print "Testing for Key: ", Key
            print "LicDict[Key]: ", LicDict[Key], len(LicDict[Key])
            # print "LicDict[Key], Tr.: ", Transmogrify(LicDict[Key])
            print "IDdict[Key]:  ", Transmogrify(IDdict[Key]), len(IDdict[Key]), len(Transmogrify(IDdict[Key]))
            File.write("Key, IDdict[Key]:  " +Key+ "\n"+Transmogrify(IDdict[Key])+ "\n"+str(len(IDdict[Key]))+ str(len(Transmogrify(IDdict[Key]))))
            if LicDict[Key] != Transmogrify(IDdict[Key]):
                File.close()
                return False
        except KeyError:
            print "KeyError for key ", Key
            File.close()
            return False

    File.close()
    return True


def GetWindowsIDs(Crypt=True):
    Lines = os.popen2("SYSTEMINFO")[1].read().split("\n") # Read system info and split into list of lines.
    IDdict = {}                                     # Will contain system info as...
    Key = ""                                        # ...key:value...
    Value = ""                                      # pairs.

    for Line in Lines:
        if Line != "" and Line[0] != " ":           # If the line contains a key (== no leading space):
            if Key != "": IDdict[Key] = Value.strip()   # Put ealier key:value pair into dictionary
            Key = Line.strip().split(":")[0]        # Use string before first ':' as key (strip leading and trailing white space.
            Value = ":".join(Line.split(":")[1:])   # The rest of the line is value.
        else:
            Value = Value + Line                    # Line with leading space: all value, append to earlier value string.

    File = open("NoitaSpinIDfile.py", "w")
    File.write("PrivIDs = [")

    for Key in LicDict.keys():
        try:
            if Crypt:
                File.write("'"+Transmogrify(IDdict[Key])+"', ")
            else:
                File.write("'"+IDdict[Key]+"', ")
        except KeyError:
             File.write("None, ")

    File.write("]\n")
    File.close()







####################################################
#from PyQt4 import QtCore

#print "hohoho", QtCore.QPointF(123.2, 123.4)

#class Test:
#    def __init__(self):
#        cdef double SpecPointsValue[1024][1024]

#X = Test()
#SpecPointsValue[0][0] = 100

